/**
  * Copyright (c): Uwe Schmidt, FH Wedel
  *
  * You may study, modify and distribute this source code
  * FOR NON-COMMERCIAL PURPOSES ONLY.
  * This copyright message has to remain unchanged.
  *
  * Note that this document is provided 'as is',
  * WITHOUT WARRANTY of any kind either expressed or implied.
  */
/**
  * The original sourcecode was modified to 
  * fit the phonebook requirements.
  * This sourcefile only contains the required 
  * functionality for the xml-example. For 
  * binary-tree-implementation, please check the
  * original sourcecode.
  * This file may contain some additional functions.
  * 
  * @author: Sascha Wille (inf6201@fh-wedel.de)
  */

#ifndef Phonebook_H__
#define Phonebook_H__

#include "entry.h"

typedef struct Node *Phonebook;

struct Node
{
  Entry info;
  Phonebook l;
  Phonebook r;
};

/**
 * returns the number of entries in the phonebook
 *
 * @param[in] pb, phonebook
 * @return no of entries
 */
int getSize(Phonebook pb);

/**
 * create an empty phonebook
 *
 * @return empty phonebook
 */
Phonebook mkEmpty(void);

/**
 * Determines wheather phonebook is empty or not
 *
 * @param[in] phonebook to check
 *
 * @return boolean 1 if phonebook is empty 0 otherwise
 */
int isEmpty(Phonebook pb);

/**
 * create a phonebook with one entry
 *
 * @param[in] entry of phonebook
 *
 * @return phonebook with exactly one entry
 */
Phonebook mkOne(Entry e);

/**
 * Determines wheather entry is in phonebook or not
 *
 * @param[in] entry to check if it's in
 * @param[in] base
 *
 * @return boolean 1 if Entry was in 0 otherwise
 */
int isIn(Entry e, Phonebook pb);

/**
 * retieves an entry due to a phonenumber
 * ASSUMPTION! Phonenumbers are unique
 *
 * @param[in] phonenumber
 * @param[in] phonebook
 *
 * @return entry if it was in; NULL otherwise
 */
Entry getEntryByPhone(char * no, Phonebook pb);

/**
 * retieves entries due to a surname
 *
 * @param[in] surname
 * @param[in] phonebook
 *
 * @return entry if it was in; NULL otherwise
 */
/*List getEntriesBySurname(char * surname);*/ /* NOT IMPLEMENTED YET */

/**
 * retieves entries due to a firstname
 *
 * @param[in] firstName
 * @param[in] phonebook
 *
 * @return entry if it was in; NULL otherwise
 */
/*List getEntriesByFirstname(char * surname);*/ /* NOT IMPLEMENTED YET */


/**
 * adds an entry into the phonebook
 *
 * @param[in] additional entry
 *
 * @return changed phonebook
 */
Phonebook addEntry(Entry e, Phonebook pb);

/**
 * removes an entry into the phonebook
 *
 * @param[in] entry to be removed
 *
 * @return changed phonebook
 */
Phonebook removeEntry(Entry e, Phonebook pb);

/**
 * Ammount of entries in a phonebook
 *
 * @param[in] phonebook to check
 *
 * @return ammount of entries
 */
unsigned int size(Phonebook pb);

/**
 * prints the whole phonebook in an
 * lexicographic way
 *
 * @param[in] pb, phonebook
 */
void printPhoneBook(Phonebook pb);

#endif


